@@ -1,8 +1,13 @@
-'''
-trimesh.py
+"""
+convex.py
 
-Library for importing and doing simple operations on triangular meshes.
-'''
+Deal with creating and checking convex objects in 2, 3 and N dimensions.
+
+Convex:
+1) "Convex, meaning "curving out" or "extending outward" (compare to concave)
+2) having an outline or surface curved like the exterior of a circle or sphere.
+3) (of a polygon) having only interior angles measuring less than 180
+"""
 
 import numpy as np
 
@@ -19,7 +24,7 @@ except ImportError:
 
 
 def convex_hull(obj, qhull_options='QbB Pp'):
-    '''
+    """
     Get a new Trimesh object representing the convex hull of the
     current mesh, with proper normals and watertight.
     Requires scipy >.12.
@@ -32,7 +37,7 @@ def convex_hull(obj, qhull_options='QbB Pp'):
     Returns
     --------
     convex: Trimesh object of convex hull
-    '''
+    """
     from .base import Trimesh
 
     if isinstance(obj, Trimesh):
@@ -114,7 +119,7 @@ def convex_hull(obj, qhull_options='QbB Pp'):
 
 
 def adjacency_projections(mesh):
-    '''
+    """
     Test if a mesh is convex by projecting the vertices of
     a triangle onto the normal of its adjacent face.
 
@@ -125,7 +130,7 @@ def adjacency_projections(mesh):
     Returns
     ----------
     projection: distance of projection of adjacent vertex onto plane
-    '''
+    """
     # normals and origins from the first column of face adjacency
     normals = mesh.face_normals[mesh.face_adjacency[:, 0]]
     # one of the vertices on the shared edge
@@ -133,11 +138,7 @@ def adjacency_projections(mesh):
 
     # faces from the second column of face adjacency
     vid_other = mesh.face_adjacency_unshared[:, 1]
-
-    vertex_other = mesh.vertices[vid_other]
-
-    vector_other = vertex_other - origins
-    length_other = (vector_other ** 2).sum(axis=1) ** .5
+    vector_other = mesh.vertices[vid_other] - origins
 
     dots = util.diagonal_dot(vector_other,
                              normals)
@@ -145,12 +146,23 @@ def adjacency_projections(mesh):
 
 
 def is_convex(mesh):
+    """
+    Check if a mesh is convex.
+
+    Parameters
+    -----------
+    mesh: Trimesh object
+
+    Returns
+    -----------
+    convex: bool, was passed mesh convex or not
+    """
     convex = (mesh.face_adjacency_projections < tol.planar).all()
     return bool(convex)
 
 
-def planar_hull(points, normal, origin=None, input_convex=False):
-    '''
+def planar_hull(points, normal, origin=None):
+    """
     Find the convex outline of a set of points projected to a plane.
 
     Parameters
@@ -158,20 +170,18 @@ def planar_hull(points, normal, origin=None, input_convex=False):
     points: (n,3) float, input points
     normal: (3) float vector, normal vector of plane
     origin: (3) float, location of plane origin
-    input_convex: bool, if True we assume the input points are already from
-                  a convex hull which provides a speedup.
 
     Returns
     -----------
     hull_lines: (n,2,2) set of unordered line segments
     T:          (4,4) float, transformation matrix
-    '''
+    height:     (2,) float, [Z min, Z max]
+    """
     from .points import project_to_plane
 
     if origin is None:
         origin = np.zeros(3)
-    if not input_convex:
-        pass
+
     planar, T = project_to_plane(points,
                                  plane_normal=normal,
                                  plane_origin=origin,
@@ -186,7 +196,7 @@ def planar_hull(points, normal, origin=None, input_convex=False):
 
 
 def hull_points(obj):
-    '''
+    """
     Try to extract a convex set of points from multiple input formats.
 
     Parameters
@@ -198,7 +208,7 @@ def hull_points(obj):
     Returns
     --------
     points: (o,d) convex set of points
-    '''
+    """
     if hasattr(obj, 'convex_hull'):
         points = obj.convex_hull.vertices
     elif util.is_sequence(obj):
