@@ -1,6 +1,9 @@
-'''
+"""
+creation.py
+--------------
+
 Create meshes from primitives, or with operations.
-'''
+"""
 
 from .base import Trimesh
 from .constants import log, tol
@@ -41,7 +44,7 @@ def validate_polygon(obj):
 def extrude_polygon(polygon,
                     height,
                     **kwargs):
-    '''
+    """
     Extrude a 2D shapely polygon into a 3D mesh
 
     Parameters
@@ -52,7 +55,7 @@ def extrude_polygon(polygon,
     Returns
     ----------
     mesh: Trimesh object of result
-    '''
+    """
     vertices, faces = triangulate_polygon(polygon, **kwargs)
     mesh = extrude_triangulation(vertices=vertices,
                                  faces=faces,
@@ -61,11 +64,114 @@ def extrude_polygon(polygon,
     return mesh
 
 
+def sweep_polygon(polygon,
+                  path,
+                  angles=None,
+                  **kwargs):
+    """
+    Extrude a 2D shapely polygon into a watertight 3D mesh
+    along an arbitrary 3D path.
+
+    Parameters
+    ----------
+    polygon: shapely.geometry.Polygon object
+    path:    (n,3) float, a path in 3D
+    angles:  (n,) float, optional rotation angle relative to prior vertex
+                         at each vertex
+
+    Returns
+    -------
+    mesh : Trimesh object of result
+    """
+    path = np.asanyarray(path, dtype=np.float64)
+    if not util.is_shape(path, (-1, 3)):
+        raise ValueError('Path must be (n, 3)!')
+
+    # Extract 2D vertices and triangulation
+    verts_2d = np.array(polygon.exterior)[:-1]
+    base_verts_2d, faces_2d = triangulate_polygon(polygon, **kwargs)
+    n = len(verts_2d)
+
+    # Create basis for first planar polygon cap
+    x, y, z = util.generate_basis(path[0] - path[1])
+    tf_mat = np.ones((4, 4))
+    tf_mat[:3, :3] = np.c_[x, y, z]
+    tf_mat[:3, 3] = path[0]
+
+    # Compute 3D locations of those vertices
+    verts_3d = np.c_[verts_2d, np.zeros(n)]
+    verts_3d = transformations.transform_points(verts_3d, tf_mat)
+    base_verts_3d = np.c_[base_verts_2d, np.zeros(len(base_verts_2d))]
+    base_verts_3d = transformations.transform_points(base_verts_3d, tf_mat)
+
+    # keep matching sequence of vertices and 0- indexed faces
+    vertices = [base_verts_3d]
+    faces = [faces_2d]
+
+    # Compute plane normals for each turn --
+    # each turn induces a plane halfway between the two vectors
+    v1s = util.unitize(path[1:-1] - path[:-2])
+    v2s = util.unitize(path[1:-1] - path[2:])
+    norms = np.cross(np.cross(v1s, v2s), v1s + v2s)
+    norms[(norms == 0.0).all(1)] = v1s[(norms == 0.0).all(1)]
+    norms = util.unitize(norms)
+    final_v1 = util.unitize(path[-1] - path[-2])
+    norms = np.vstack((norms, final_v1))
+    v1s = np.vstack((v1s, final_v1))
+
+    # Create all side walls by projecting the 3d vertices into each plane
+    # in succession
+    for i in range(len(norms)):
+        verts_3d_prev = verts_3d
+
+        # Rotate if needed
+        if angles is not None:
+            tf_mat = transformations.rotation_matrix(angles[i],
+                                                     norms[i],
+                                                     path[i])
+            verts_3d_prev = transformations.transform_points(verts_3d_prev,
+                                                             tf_mat)
+
+        # Project vertices onto plane in 3D
+        ds = np.einsum('ij,j->i', (path[i + 1] - verts_3d_prev), norms[i])
+        ds = ds / np.dot(v1s[i], norms[i])
+
+        verts_3d_new = np.einsum('i,j->ij', ds, v1s[i]) + verts_3d_prev
+
+        # Add to face and vertex lists
+        new_faces = [[i + n, (i + 1) % n, i] for i in range(n)]
+        new_faces.extend([[(i - 1) % n + n, i + n, i] for i in range(n)])
+
+        # save faces and vertices into a sequence
+        faces.append(np.array(new_faces))
+        vertices.append(np.vstack((verts_3d, verts_3d_new)))
+
+        verts_3d = verts_3d_new
+
+    # do the main stack operation from a sequence to (n,3) arrays
+    # doing one vstack provides a substantial speedup by
+    # avoiding a bunch of temporary  allocations
+    vertices, faces = util.append_faces(vertices, faces)
+
+    # Create final cap
+    x, y, z = util.generate_basis(path[-1] - path[-2])
+    vecs = verts_3d - path[-1]
+    coords = np.c_[np.einsum('ij,j->i', vecs, x),
+                   np.einsum('ij,j->i', vecs, y)]
+    base_verts_2d, faces_2d = triangulate_polygon(Polygon(coords))
+    base_verts_3d = (np.einsum('i,j->ij', base_verts_2d[:, 0], x) +
+                     np.einsum('i,j->ij', base_verts_2d[:, 1], y)) + path[-1]
+    faces = np.vstack((faces, faces_2d + len(vertices)))
+    vertices = np.vstack((vertices, base_verts_3d))
+
+    return Trimesh(vertices, faces)
+
+
 def extrude_triangulation(vertices,
                           faces,
                           height,
                           **kwargs):
-    '''
+    """
     Turn a shapely.geometry Polygon object and a height (float)
     into a watertight Trimesh object.
 
@@ -78,7 +184,7 @@ def extrude_triangulation(vertices,
     Returns
     ---------
     mesh: Trimesh object of result
-    '''
+    """
     vertices = np.asanyarray(vertices, dtype=np.float64)
     faces = np.asanyarray(faces, dtype=np.int)
     height = float(height)
@@ -141,8 +247,9 @@ def extrude_triangulation(vertices,
 
 
 def triangulate_polygon(polygon, **kwargs):
-    '''
-    Given a shapely polygon, create a triangulation using meshpy.triangle
+    """
+    Given a shapely polygon create a triangulation using
+    meshpy.triangle
 
     Parameters
     ---------
@@ -163,18 +270,51 @@ def triangulate_polygon(polygon, **kwargs):
     Returns
     --------
     mesh_vertices: (n, 2) float array of 2D points
-    mesh_faces:    (n, 3) int array of vertex indicies representing triangles
-    '''
+    mesh_faces:    (n, 3) int array of vertex indexes
+    """
+    # do the import here, as sometimes this import can segfault
+    # which is not catchable with a try/except block
+    import meshpy.triangle as triangle
+
+    # turn the polygon in to vertices, segments, and hole points
+    arg = _polygon_to_kwargs(polygon)
+    # call meshpy.triangle on our cleaned representation of
+    # the Shapely polygon
+    info = triangle.MeshInfo()
+    info.set_points(arg['vertices'])
+    info.set_facets(arg['segments'])
+    info.set_holes(arg['holes'])
+
+    # build mesh and pass kwargs to triangle
+    mesh = triangle.build(info, **kwargs)
+
+    # (n, 2) float vertices
+    vertices = np.array(mesh.points, dtype=np.float64)
+    # (m, 3) int faces
+    faces = np.array(mesh.elements, dtype=np.int64)
+
+    return vertices, faces
+
+
+def _polygon_to_kwargs(polygon):
+    """
+    Given a shapely polygon generate the data to pass to
+    the triangle mesh generator
+
+    Parameters
+    ---------
+    polygon: Shapely.geometry.Polygon
+
+    Returns
+    --------
+    result: dict, with keys: vertices, segments, holes
+    """
 
     if not polygon.is_valid:
         raise ValueError('invalid shapely polygon passed!')
 
-    # do the import here, as sometimes this import can segfault python
-    # which is not catchable with a try/except block
-    import meshpy.triangle as triangle
-
     def round_trip(start, length):
-        '''
+        """
         Given a start index and length, create a series of (n, 2) edges which
         create a closed traversal.
 
@@ -182,7 +322,7 @@ def triangulate_polygon(polygon, **kwargs):
         ---------
         start, length = 0, 3
         returns:  [(0,1), (1,2), (2,0)]
-        '''
+        """
         tiled = np.tile(np.arange(start, start + length).reshape((-1, 1)), 2)
         tiled = tiled.reshape(-1)[1:-1].reshape((-1, 2))
         tiled = np.vstack((tiled, [tiled[-1][-1], tiled[0][0]]))
@@ -238,25 +378,26 @@ def triangulate_polygon(polygon, **kwargs):
     # we added a hole for the exterior, which we slice away here
     holes = np.array(holes)[1:]
 
-    # call meshpy.triangle on our cleaned representation of the Shapely polygon
-    info = triangle.MeshInfo()
-    info.set_points(vertices)
-    info.set_facets(facets)
-    info.set_holes(holes)
+    result = {'vertices': vertices,
+              'segments': facets,
+              'holes': holes}
 
-    # uses kwargs
-    mesh = triangle.build(info, **kwargs)
-
-    mesh_vertices = np.array(mesh.points)
-    mesh_faces = np.array(mesh.elements)
-
-    return mesh_vertices, mesh_faces
+    return result
 
 
 def box(extents=None, transform=None):
-    '''
-    Return a unit cube, centered at the origin with edges of length 1.0
-    '''
+    """
+    Return a cuboid.
+
+    Parameters
+    ------------
+    extents: float, or (3,) float edge length
+    transform: (4, 4) float, transformation matrix
+
+    Returns
+    ------------
+    box: Trimesh object
+    """
     vertices = [0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1,
                 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1]
     vertices = np.array(vertices,
@@ -292,10 +433,10 @@ def box(extents=None, transform=None):
 
 
 def icosahedron():
-    '''
+    """
     Create an icosahedron, a 20 faced polyhedron.
 
-    '''
+    """
     t = (1.0 + 5.0**.5) / 2.0
     vertices = [-1, t, 0, 1, t, 0, -1, -t, 0, 1, -t, 0, 0, -1, t, 0, 1, t,
                 0, -1, -t, 0, 1, -t, t, 0, -1, t, 0, 1, -t, 0, -1, -t, 0, 1]
@@ -313,7 +454,7 @@ def icosahedron():
 
 
 def icosphere(subdivisions=3, radius=1.0):
-    '''
+    """
     Create an isophere centered at the origin.
 
     Parameters
@@ -326,7 +467,7 @@ def icosphere(subdivisions=3, radius=1.0):
     Returns
     ---------
     ico: trimesh.Trimesh object of sphere
-    '''
+    """
     def refine_spherical():
         vectors = ico.vertices
         scalar = (vectors ** 2).sum(axis=1)**.5
@@ -346,7 +487,7 @@ def uv_sphere(radius=1.0,
               count=[32, 32],
               theta=None,
               phi=None):
-    '''
+    """
     Create a UV sphere (latitude + longitude) centered at the origin.
 
     Roughly one order of magnitude faster than an icosphere but slightly uglier.
@@ -361,7 +502,7 @@ def uv_sphere(radius=1.0,
     Returns
     ----------
     mesh: Trimesh object of UV sphere with specified parameters
-    '''
+    """
 
     count = np.array(count, dtype=np.int)
     count += np.mod(count, 2)
@@ -414,7 +555,7 @@ def uv_sphere(radius=1.0,
 def capsule(height=1.0,
             radius=1.0,
             count=[32, 32]):
-    '''
+    """
     Create a mesh of a capsule, or a cylinder with hemispheric ends.
 
     Parameters
@@ -429,7 +570,7 @@ def capsule(height=1.0,
              - cylinder axis is along Z
              - one hemisphere is centered at the origin
              - other hemisphere is centered along the Z axis at specified height
-    '''
+    """
     height = float(height)
     radius = float(radius)
     count = np.array(count, dtype=np.int)
@@ -456,7 +597,7 @@ def capsule(height=1.0,
 
 
 def cylinder(radius=1.0, height=1.0, sections=32, transform=None):
-    '''
+    """
     Create a mesh of a cylinder along Z centered at the origin.
 
     Parameters
@@ -468,7 +609,7 @@ def cylinder(radius=1.0, height=1.0, sections=32, transform=None):
     Returns
     ----------
     cylinder: Trimesh, resulting mesh
-    '''
+    """
 
     # create a 2D pie out of wedges
     theta = np.linspace(0, np.pi * 2, sections)
@@ -500,7 +641,7 @@ def cylinder(radius=1.0, height=1.0, sections=32, transform=None):
 
 
 def random_soup(face_count=100):
-    '''
+    """
     Return a random set of triangles as a Trimesh
 
     Parameters
@@ -510,7 +651,7 @@ def random_soup(face_count=100):
     Returns
     -----------
     soup: Trimesh object with face_count random faces
-    '''
+    """
     vertices = np.random.random((face_count * 3, 3)) - 0.5
     faces = np.arange(face_count * 3).reshape((-1, 3))
     soup = Trimesh(vertices=vertices, faces=faces)
