@@ -12,7 +12,7 @@ except ImportError:
 
 
 def plane_transform(origin, normal):
-    '''
+    """
     Given the origin and normal of a plane, find the transform that will move
     that plane to be coplanar with the XY plane
 
@@ -24,37 +24,14 @@ def plane_transform(origin, normal):
     Returns
     ---------
     transform: (4,4) float, transformation matrix
-    '''
+    """
     transform = align_vectors(normal, [0, 0, 1])
     transform[0:3, 3] = -np.dot(transform, np.append(origin, 1))[0:3]
     return transform
 
 
-def transform_around(matrix, point):
-    '''
-    Given a transformation matrix, apply its rotation component around a
-    point in space.
-
-    Parameters
-    ----------
-    matrix: (4,4) float, transformation matrix
-    point:  (3,)  float, point in space
-
-    Returns
-    ---------
-    result: (4,4) transformation matrix
-    '''
-    point = np.array(point)
-    translate = np.eye(4)
-    translate[0:3, 3] = -point
-    result = np.dot(matrix, translate)
-    translate[0:3, 3] = point
-    result = np.dot(translate, result)
-    return result
-
-
 def align_vectors(vector_start, vector_end, return_angle=False):
-    '''
+    """
     Returns the 4x4 transformation matrix which will rotate from
     vector_start to vector_end, eg:
 
@@ -72,7 +49,7 @@ def align_vectors(vector_start, vector_end, return_angle=False):
     transform: (4,4) float, transformation matrix
     angle:     float, angle in radians (only returned if flag set)
 
-    '''
+    """
     start = np.asanyarray(vector_start, dtype=np.float64)
     start /= np.linalg.norm(start)
     end = np.asanyarray(vector_end, dtype=np.float64)
@@ -90,6 +67,7 @@ def align_vectors(vector_start, vector_end, return_angle=False):
         # and no rotation is needed
         T = np.eye(4)
         T[0:3] *= direction
+        angle = 0.0
     else:
         angle = np.arcsin(norm)
         if direction < 0:
@@ -106,7 +84,7 @@ def align_vectors(vector_start, vector_end, return_angle=False):
 
 
 def faces_to_edges(faces, return_index=False):
-    '''
+    """
     Given a list of faces (n,3), return a list of edges (n*3,2)
 
     Parameters
@@ -116,7 +94,7 @@ def faces_to_edges(faces, return_index=False):
     Returns
     -----------
     edges: (n*3, 2) int, vertex indices representing edges
-    '''
+    """
     faces = np.asanyarray(faces)
     edges = np.column_stack((faces[:, (0, 1)],
                              faces[:, (1, 2)],
@@ -128,7 +106,7 @@ def faces_to_edges(faces, return_index=False):
 
 
 def vector_angle(pairs):
-    '''
+    """
     Find the angles between vector pairs
 
     Parameters
@@ -142,7 +120,7 @@ def vector_angle(pairs):
     Examples
     ----------
     angles = mesh.face_normals[mesh.face_adjacency]
-    '''
+    """
     pairs = np.asanyarray(pairs)
     if len(pairs) == 0:
         return np.array([])
@@ -156,7 +134,7 @@ def vector_angle(pairs):
 
 
 def triangulate_quads(quads):
-    '''
+    """
     Given a set of quad faces, return them as triangle faces.
 
     Parameters
@@ -166,7 +144,7 @@ def triangulate_quads(quads):
     Returns
     -----------
     faces: (m,3) int, vertex indices of triangular faces
-    '''
+    """
     if len(quads) == 0:
         return quads
     quads = np.asanyarray(quads)
@@ -176,7 +154,7 @@ def triangulate_quads(quads):
 
 
 def mean_vertex_normals(vertex_count, faces, face_normals, **kwargs):
-    '''
+    """
     Find vertex normals from the mean of the faces that contain that vertex.
 
     Parameters
@@ -189,7 +167,7 @@ def mean_vertex_normals(vertex_count, faces, face_normals, **kwargs):
     -----------
     vertex_normals: (vertex_count, 3) float normals for every vertex
                     Uncontained vertices will be zero.
-    '''
+    """
     def summed_sparse():
         # use a sparse matrix of which face contains each vertex to
         # figure out the summed normal at each vertex
@@ -224,7 +202,7 @@ def mean_vertex_normals(vertex_count, faces, face_normals, **kwargs):
 
 
 def index_sparse(column_count, indices):
-    '''
+    """
     Return a sparse matrix for which vertices are contained in which faces.
 
     Returns
@@ -264,7 +242,7 @@ def index_sparse(column_count, indices):
 
     In [7]: dense.sum(axis=0)
     Out[7]: array([3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3])
-    '''
+    """
     indices = np.asanyarray(indices)
     column_count = int(column_count)
 
@@ -278,36 +256,3 @@ def index_sparse(column_count, indices):
                         shape=shape,
                         dtype=np.bool)
     return sparse
-
-
-def medial_axis(samples, contains):
-    '''
-    Given a set of samples on a boundary, find the approximate medial axis based
-    on a voronoi diagram and a containment function which can assess whether
-    a point is inside or outside of the closed geometry.
-
-    Parameters
-    ----------
-    samples:    (n,d) set of points on the boundary of the geometry
-    contains:   function which takes (m,d) points and returns an (m) bool array
-
-    Returns
-    ----------
-    lines:     (n,2,2) set of line segments
-    '''
-
-    from scipy.spatial import Voronoi
-    from .path.io.load import load_path
-
-    # create the voronoi diagram, after vertically stacking the points
-    # deque from a sequnce into a clean (m,2) array
-    voronoi = Voronoi(samples)
-    # which voronoi vertices are contained inside the original polygon
-    contained = contains(voronoi.vertices)
-    # ridge vertices of -1 are outside, make sure they are False
-    contained = np.append(contained, False)
-    inside = [i for i in voronoi.ridge_vertices if contained[i].all()]
-    line_indices = np.vstack([util.stack_lines(i)
-                              for i in inside if len(i) >= 2])
-    lines = voronoi.vertices[line_indices]
-    return load_path(lines)
