@@ -10,7 +10,7 @@ from ... import util
 
 
 def load_path(obj, file_type=None):
-    '''
+    """
     Load a file to a Path object.
 
     Parameters
@@ -32,7 +32,7 @@ def load_path(obj, file_type=None):
     Returns
     ---------
     path: Path2D or Path3D object
-    '''
+    """
 
     if isinstance(obj, Path):
         return obj
@@ -58,15 +58,17 @@ def load_path(obj, file_type=None):
 
 
 def _create_path(entities, vertices, metadata=None, **kwargs):
-    shape = np.shape(vertices)
-    if ((len(shape) != 2) or
-            (not shape[1] in [2, 3])):
-        raise ValueError('Vertices must be 2D or 3D!')
-    path = [Path2D, Path3D][shape[1] == 3](entities=entities,
-                                           vertices=vertices,
-                                           metadata=metadata,
-                                           **kwargs)
-    return path
+    vertices = np.asanyarray(vertices)
+
+    if len(vertices.shape) != 2:
+        path_type = Path
+    else:
+        path_type = [Path2D, Path3D][int(vertices.shape[1] == 3)]
+
+    return path_type(entities=entities,
+                     vertices=vertices,
+                     metadata=metadata,
+                     **kwargs)
 
 
 def path_formats():
