@@ -12,8 +12,12 @@ from ..constants import log
 from ..constants import tol_path as tol
 
 
-def fit_circle_check(points, scale, prior=None, final=False, verbose=False):
-    '''
+def fit_circle_check(points,
+                     scale,
+                     prior=None,
+                     final=False,
+                     verbose=False):
+    """
     Fit a circle, and reject the fit if:
     * the radius is larger than tol.radius_min*scale or tol.radius_max*scale
     * any segment spans more than tol.seg_angle
@@ -36,7 +40,7 @@ def fit_circle_check(points, scale, prior=None, final=False, verbose=False):
         (center, radius) tuple
     else:
         None
-    '''
+    """
     # an arc needs at least three points
     if len(points) < 3:
         return None
@@ -94,14 +98,28 @@ def fit_circle_check(points, scale, prior=None, final=False, verbose=False):
                       np.degrees(tangent))
         return None
 
-    return (C, R)
+    result = {'center': C,
+              'radius': R}
+
+    return result
 
 
 def is_circle(points, scale, verbose=False):
-    '''
+    """
     Given a set of points, quickly determine if they represent
     a circle or not.
-    '''
+
+    Parameters
+    -------------
+    points: (n,2) float, points in space
+    scale:  float, scale of overall drawing
+    verbose: bool, print all fit messages or not
+
+    Returns
+    -------------
+    control: (3,2) float, points in space, OR
+              None, if not a circle
+    """
 
     # make sure input is a numpy array
     points = np.asanyarray(points)
@@ -125,12 +143,12 @@ def is_circle(points, scale, verbose=False):
         return None
 
     # return the circle as three control points
-    control = arc.angles_to_threepoint([0, np.pi * .5], *CR)
+    control = arc.to_threepoint(**CR)
     return control
 
 
-def merge_colinear(points, scale=None):
-    '''
+def merge_colinear(points, scale):
+    """
     Given a set of points representing a path in space,
     merge points which are colinear.
 
@@ -143,10 +161,9 @@ def merge_colinear(points, scale=None):
     ----------
     merged: (j, d) set of points with colinear and duplicate
              points merged, where (j < n)
-    '''
-    points = np.array(points)
-    if scale is None:
-        scale = np.ptp(points, axis=0).max()
+    """
+    points = np.asanyarray(points)
+    scale = float(scale)
 
     # the vector from one point to the next
     direction = points[1:] - points[:-1]
@@ -183,7 +200,7 @@ def merge_colinear(points, scale=None):
 
 
 def resample_spline(points, smooth=.001, count=None, degree=3):
-    '''
+    """
     Resample a path in space, smoothing along a b-spline.
 
     Parameters
@@ -196,7 +213,7 @@ def resample_spline(points, smooth=.001, count=None, degree=3):
     Returns
     ---------
     resampled: (count, dimension) float, points in space
-    '''
+    """
     from scipy.interpolate import splprep, splev
     if count is None:
         count = len(points)
@@ -216,7 +233,7 @@ def resample_spline(points, smooth=.001, count=None, degree=3):
 
 
 def points_to_spline_entity(points, smooth=.0005, count=None):
-    '''
+    """
     Create a spline entity from a curve in space
 
     Parameters
@@ -229,7 +246,7 @@ def points_to_spline_entity(points, smooth=.0005, count=None):
     ---------
     entity: entities.BSpline object with points indexed at zero
     control: (m, dimension) float, new vertices for entity
-    '''
+    """
 
     from scipy.interpolate import splprep
     if count is None:
@@ -254,7 +271,7 @@ def points_to_spline_entity(points, smooth=.0005, count=None):
 
 
 def three_point(indices):
-    '''
+    """
     Given a long list of ordered indices,
     return the first, middle and last.
 
@@ -265,27 +282,28 @@ def three_point(indices):
     Returns
     ----------
     three: (3,) array
-    '''
+    """
     three = [indices[0],
              indices[int(len(indices) / 2)],
              indices[-1]]
-    return np.array(three)
+    return np.asanyarray(three)
 
 
-def simplify_basic(drawing):
-    '''
+def simplify_basic(drawing, process=False, **kwargs):
+    """
     Merge colinear segments and fit circles.
 
     Parameters
     -----------
-    drawing: Path2D object
+    drawing: Path2D object, will not be modified.
 
     Returns
     -----------
     simplified: Path2D with circles.
-    '''
+    """
 
-    if any(i.__class__.__name__ != 'Line' for i in drawing.entities):
+    if any(i.__class__.__name__ != 'Line'
+           for i in drawing.entities):
         log.debug('Path contains non- linear entities, skipping')
         return drawing
 
@@ -297,18 +315,14 @@ def simplify_basic(drawing):
     vertices_new = collections.deque()
     entities_new = collections.deque()
 
-    for polygon in drawing.polygons_closed:
-
-        # clean up things like self intersections
-        buffered = polygon.buffer(0.0)
-        # get the exterior as an (n,2) array
-        # since we generated these from the closed
-        points = merge_colinear(np.array(buffered.exterior.coords),
-                                scale=drawing.scale)
-        # check to see if the closed entity represents a circle
-        circle = is_circle(points,
-                           scale=drawing.scale)
+    # avoid thrashing cache in loop
+    scale = drawing.scale
 
+    # loop through (n, 2) closed paths
+    for discrete in drawing.discrete:
+        # check to see if the closed entity is a circle
+        circle = is_circle(discrete,
+                           scale=scale)
         if circle is not None:
             # the points are circular enough for our high standards
             # so replace them with a closed Arc entity
@@ -317,25 +331,87 @@ def simplify_basic(drawing):
                                              closed=True))
             vertices_new.extend(circle)
         else:
-            # save this path as a closed Line entity
-            # we cleaned up colinear points so it will still
-            # be simpler than the source data
+            # not a circle, so clean up colinear segments
+            # then save it as a single line entity
+            points = merge_colinear(discrete, scale=scale)
+            # references for new vertices
             indexes = np.arange(len(points)) + len(vertices_new)
+            # discrete curves are always closed
+            indexes[-1] = indexes[0]
+            # append new vertices and entity
             entities_new.append(entities.Line(points=indexes))
             vertices_new.extend(points)
 
     # create the new drawing object
-    simplified = type(drawing)(entities=entities_new,
-                               vertices=vertices_new)
-
+    simplified = type(drawing)(
+        entities=entities_new,
+        vertices=vertices_new,
+        metadata=copy.deepcopy(drawing.metadata),
+        process=process)
     # we have changed every path to a single closed entity
     # either a closed arc, or a closed line
-    # therefore all closed paths are now represented by a single entity
-    cache.cache.update({'paths': np.arange(len(entities_new)).reshape((-1, 1)),
-                        'path_valid': np.ones(len(entities_new), dtype=np.bool),
-                        'dangling': np.array([])})
+    # so all closed paths are now represented by a single entity
+    cache.cache.update({
+        'paths': np.arange(len(entities_new)).reshape((-1, 1)),
+        'path_valid': np.ones(len(entities_new), dtype=np.bool),
+        'dangling': np.array([])})
+
+    # force recompute of exact bounds
+    if 'bounds' in cache.cache:
+        cache.cache.pop('bounds')
+
     simplified._cache = cache
     # set the cache ID so it won't dump when a value is requested
     simplified._cache.id_set()
 
     return simplified
+
+
+def simplify_spline(path, smooth, path_indexes=None):
+    """
+    Replace discrete curves with b-spline curves, and
+    return the result as a new Path2D object.
+
+    Parameters
+    ------------
+    path:         Path2D object
+    smooth:       float, amount to smooth
+    path_indexes: (n,) int, indexes of path.paths to simplify
+
+    Returns
+    ------------
+    simplified: Path2D object, with specified entities replaced
+    """
+    # if we aren't simplifying specific indexes
+    # simplify all indexes in the path object
+    if path_indexes is None:
+        path_indexes = np.arange(len(path.paths))
+
+    entities_keep = np.ones(len(path.entities),
+                            dtype=np.bool)
+    new_vertices = []
+    new_entities = []
+
+    for i in path_indexes:
+        # entities for this path
+        entity, vertices = points_to_spline_entity(path.discrete[i])
+        # reindex returned control points
+        entity.points += len(path.vertices) + len(new_vertices)
+        # save entity and vertices
+        new_vertices.append(vertices)
+        new_entities.append(entity)
+        # we don't need any of the entities from the
+        # path we just consumed and replaced
+        entities_keep[path.paths[i]] = False
+
+    # flatten entities and vertices
+    entities = np.append(path.entities[entities_keep],
+                         new_entities)
+    vertices = np.vstack((path.vertices,
+                          np.vstack(new_vertices)))
+
+    # create the Path2D object for the result
+    simplified = type(path)(entities=entities,
+                            vertices=vertices)
+
+    return simplified
