@@ -337,9 +337,20 @@ class PointCloud(object):
         return self._data.md5()
 
     def merge_vertices(self):
+        """
+        Merge vertices closer than tol.merge (default: 1e-8)
+        """
+        # run unique rows
         unique, inverse = grouping.unique_rows(self.vertices)
+
+        # apply unique mask to vertices
         self.vertices = self.vertices[unique]
 
+        # apply unique mask to colors
+        if (self.colors is not None and
+                len(self.colors) == len(inverse)):
+            self.colors = self.colors[unique]
+
     @property
     def bounds(self):
         return np.array([self.vertices.min(axis=0),
