@@ -146,21 +146,49 @@ def closest_point(mesh, points):
 
     distance_2 = ((query_close - query_point) ** 2).sum(axis=1)
 
-    # find the single closest point f6or each group of candidates
+    # find the single closest point for each group of candidates
     result_close = np.zeros((len(points), 3), dtype=np.float64)
     result_tid = np.zeros(len(points), dtype=np.int64)
     result_distance = np.zeros(len(points), dtype=np.float64)
 
+    # go through results to get minimum distance result
     for i, close_points, distance, candidate in zip(
-        np.arange(
-            len(points)), np.array_split(
-            query_close, query_group), np.array_split(
-                distance_2, query_group), candidates):
+            np.arange(len(points)),
+            np.array_split(query_close, query_group),
+            np.array_split(distance_2, query_group),
+            candidates):
+
+        # unless some other check is true use the smallest distance
         idx = distance.argmin()
+
+        # if we have multiple candidates check them
+        if len(candidate) > 1:
+            # (2, ) int, list of 2 closest candidate indices
+            idxs = distance.argsort()[:2]
+            # make sure the two distances are identical
+            check_distance = distance[idxs].ptp() < tol.merge
+            # make sure the magnitude of both distances are nonzero
+            check_magnitude = (np.abs(distance[idxs]) > tol.merge).all()
+
+            # check if query-points are actually off-surface
+            if check_distance and check_magnitude:
+                # get face normals for two points
+                normals = mesh.face_normals[np.array(candidate)[idxs]]
+                # compute normalized surface-point to query-point vectors
+                vectors = ((points[i] - close_points[idxs]) /
+                           distance[idxs, np.newaxis] ** 0.5)
+                # compare enclosed angle for both face normals
+                dots = util.diagonal_dot(normals, vectors)
+                # take the idx with the most positive angle
+                idx = idxs[dots.argmax()]
+
+        # take the single closest value from the group of values
         result_close[i] = close_points[idx]
         result_tid[i] = candidate[idx]
         result_distance[i] = distance[idx]
-    # we were comparing the distance squared, so now take the square root
+
+    # we were comparing the distance squared so
+    # now take the square root in one vectorized operation
     result_distance **= .5
 
     return result_close, result_distance, result_tid
