@@ -6,7 +6,7 @@ from .. import constants
 
 
 def contains_points(intersector, points, check_direction=None):
-    '''
+    """
     Check if a mesh contains a set of points, using ray tests.
 
     If the point is on the surface of the mesh, behavior is undefined.
@@ -19,7 +19,7 @@ def contains_points(intersector, points, check_direction=None):
     Returns
     ---------
     contains: (n) boolean array, whether point is inside mesh or not
-    '''
+    """
 
     # convert points to float and make sure they are 3D
     points = np.asanyarray(points, dtype=np.float64)
@@ -53,10 +53,10 @@ def contains_points(intersector, points, check_direction=None):
                                  (inside_aabb.sum(), 1))
 
     # cast a ray both forwards and backwards
-    index_ray = intersector.intersects_location(np.vstack((points[inside_aabb],
-                                                           points[inside_aabb])),
-                                                np.vstack((ray_directions,
-                                                           -ray_directions)))[1]
+    index_ray = intersector.intersects_location(
+        np.vstack(
+            (points[inside_aabb], points[inside_aabb])), np.vstack(
+            (ray_directions, -ray_directions)))[1]
 
     # if we hit nothing in either direction just return with no hits
     if len(index_ray) == 0:
@@ -112,7 +112,8 @@ def contains_points(intersector, points, check_direction=None):
         contains[mask] = contains_points(intersector,
                                          points[inside_aabb][broken],
                                          check_direction=new_direction)
-        constants.log.debug('detected %d broken contains test, attempted to fix',
-                            broken.sum())
+        constants.log.debug(
+            'detected %d broken contains test, attempted to fix',
+            broken.sum())
 
     return contains
