@@ -5,6 +5,7 @@ import copy
 from .. import util
 from .. import units
 from .. import convex
+from .. import caching
 from .. import grouping
 from .. import transformations
 
@@ -34,7 +35,7 @@ class Scene:
         # graph structure of instances
         self.graph = TransformForest(base_frame=base_frame)
 
-        self._cache = util.Cache(id_function=self.md5)
+        self._cache = caching.Cache(id_function=self.md5)
 
         self.add_geometry(geometry)
         self.set_camera()
@@ -124,6 +125,41 @@ class Scene:
         is_empty = len(self.geometry) == 0
         return is_empty
 
+    @util.cache_decorator
+    def bounds_corners(self):
+        """
+        A list of points that represent the corners of the
+        AABB of every geometry in the scene.
+
+        This can be useful if you want to take the AABB in
+        a specific frame.
+
+        Returns
+        -----------
+        corners: (n, 3) float, points in space
+        """
+        # the saved corners of each instance
+        corners_inst = []
+        # (n, 3) float corners of each geometry
+        corners_geom = {k: bounds_module.corners(v.bounds)
+                        for k, v in self.geometry.items()}
+
+        for node_name in self.graph.nodes_geometry:
+            # access the transform and geometry name from node
+            transform, geometry_name = self.graph[node_name]
+            # not all nodes have associated geometry
+            if geometry_name is None:
+                continue
+            # transform geometry corners into where
+            # the instance of the geometry is located
+            corners_inst.extend(
+                transformations.transform_points(
+                    corners_geom[geometry_name],
+                    transform))
+        # make corners numpy array
+        corners_inst = np.array(corners_inst, dtype=np.float64)
+        return corners_inst
+
     @util.cache_decorator
     def bounds(self):
         '''
@@ -133,24 +169,7 @@ class Scene:
         --------
         bounds: (2,3) float points for min, max corner
         '''
-        corners = collections.deque()
-
-        for node_name in self.graph.nodes_geometry:
-            # access the transform and geometry name for every node
-            transform, geometry_name = self.graph[node_name]
-
-            # not all nodes have associated geometry
-            if geometry_name is None:
-                continue
-
-            # geometry objects have bounds properties, which are (2,3) or (2,2)
-            current_bounds = self.geometry[geometry_name].bounds.copy()
-            # find the 8 corner vertices of the axis aligned bounding box
-            current_corners = bounds_module.corners(current_bounds)
-            # transform those corners into where the geometry is located
-            corners.extend(transformations.transform_points(current_corners,
-                                                            transform))
-        corners = np.array(corners)
+        corners = self.bounds_corners
         bounds = np.array([corners.min(axis=0),
                            corners.max(axis=0)])
         return bounds
@@ -231,6 +250,19 @@ class Scene:
         populate = self.triangles
         return self._cache['triangles_node']
 
+    @util.cache_decorator
+    def geometry_identifiers(self):
+        '''
+        Look up geometries by identifier MD5
+
+        Returns
+        ---------
+        identifiers: dict, identifier md5: key in self.geometry
+        '''
+        identifiers = {mesh.identifier_md5: name
+                       for name, mesh in self.geometry.items()}
+        return identifiers
+
     @util.cache_decorator
     def duplicate_nodes(self):
         '''
@@ -377,9 +409,10 @@ class Scene:
         '''
         from .. import primitives
         center = self.bounds.mean(axis=0)
-        aabb = primitives.Box(transform=transformations.translation_matrix(center),
-                              extents=self.extents,
-                              mutable=False)
+        aabb = primitives.Box(
+            transform=transformations.translation_matrix(center),
+            extents=self.extents,
+            mutable=False)
         return aabb
 
     @util.cache_decorator
@@ -463,7 +496,7 @@ class Scene:
 
         Parameters
         -----------
-        resultion: (2,) int, resolution to render image
+        resolution: (2,) int, resolution to render image
         **kwargs:  passed to SceneViewer constructor
 
         Returns
@@ -498,18 +531,19 @@ class Scene:
 
         current = existing[0]
         if current is None:
-            if guess:
-                current = units.unit_guess(self.scale)
-            else:
-                raise ValueError('units not defined and not allowed to guess!')
-
-        # exit early if our current units are the same as desired units
-        if current == desired:
-            return self
+            # will raise ValueError if not in metadata
+            # and not allowed to guess
+            current = units.units_from_metadata(self, guess=guess)
 
+        # find the float conversion
         scale = units.unit_conversion(current=current,
                                       desired=desired)
-        result = self.scaled(scale=scale)
+
+        # exit early if our current units are the same as desired units
+        if np.isclose(scale, 1.0):
+            result = self.copy()
+        else:
+            result = self.scaled(scale=scale)
 
         for geometry in result.geometry.values():
             geometry.units = desired
@@ -533,7 +567,6 @@ class Scene:
         vector = np.asanyarray(vector, dtype=np.float64)
         origin = np.asanyarray(origin, dtype=np.float64)
 
-        centroids = collections.deque()
         for node_name in self.graph.nodes_geometry:
             transform, geometry_name = self.graph[node_name]
 
@@ -576,12 +609,15 @@ class Scene:
 
         for group in grouping.group(geometries):
             geometry = geometries[group[0]]
-
             original = transforms[group[0]]
             new_geom = np.dot(scale_matrix, original)
-            inv_geom = np.linalg.inv(new_geom)
 
-            result.geometry[geometry].apply_transform(new_geom)
+            if result.geometry[geometry].vertices.shape[1] == 2:
+                # if our scene is 2D only scale in 2D
+                result.geometry[geometry].apply_transform(np.eye(3) * scale)
+            else:
+                # otherwise apply the full transform
+                result.geometry[geometry].apply_transform(new_geom)
 
             for node, t in zip(self.graph.nodes_geometry[group],
                                transforms[group]):
@@ -589,7 +625,6 @@ class Scene:
                                             t,
                                             np.linalg.inv(new_geom)])
                 transform[:3, 3] *= scale
-
                 result.graph.update(frame_to=node,
                                     matrix=transform,
                                     geometry=geometry)
@@ -622,7 +657,8 @@ class Scene:
 
 def split_scene(geometry):
     '''
-    Given a possible sequence of geometries, decompose them into parts.
+    Given a geometry, list of geometries, or a Scene
+    return them as a single Scene object.
 
     Parameters
     ----------
@@ -632,14 +668,26 @@ def split_scene(geometry):
     ---------
     scene: trimesh.Scene
     '''
+    # already a scene, so return it
     if util.is_instance_named(geometry, 'Scene'):
         return geometry
 
+    # a list of things
     if util.is_sequence(geometry):
-        return Scene(geometry)
+        metadata = {}
+        for g in geometry:
+            try:
+                metadata.update(g.metadata)
+            except BaseException:
+                continue
+        return Scene(geometry,
+                     metadata=metadata)
 
+    # a single geometry so we are going to split
     split = collections.deque()
+    metadata = {}
     for g in util.make_sequence(geometry):
         split.extend(g.split())
-    scene = Scene(split)
+        metadata.update(g.metadata)
+    scene = Scene(split, metadata=metadata)
     return scene
