@@ -5,6 +5,7 @@ import numpy as np
 import networkx as nx
 
 from .. import util
+from .. import caching
 from .. import transformations
 
 
@@ -17,13 +18,13 @@ class TransformForest:
         self._paths = {}
         self._updated = time.time()
 
-        self._cache = util.Cache(id_function=self.md5)
+        self._cache = caching.Cache(id_function=self.md5)
 
     def update(self,
                frame_to,
                frame_from=None,
                **kwargs):
-        '''
+        """
         Update a transform in the tree.
 
         Parameters
@@ -41,7 +42,7 @@ class TransformForest:
         translation: (3) array
 
         geometry: Geometry object name
-        '''
+        """
         if frame_from is None:
             frame_from = self.base_frame
         matrix = kwargs_to_matrix(**kwargs)
@@ -64,18 +65,18 @@ class TransformForest:
         self._updated = time.time()
 
     def md5(self):
-        '''
+        """
         MD5 of transforms.
 
         Currently only hashing update time.
-        '''
+        """
         result = str(int(self._updated * 1000)) + str(self.base_frame)
         return result
 
     def to_flattened(self, base_frame=None):
-        '''
+        """
         Export the current transform graph as a flattened
-        '''
+        """
         if base_frame is None:
             base_frame = self.base_frame
 
@@ -90,38 +91,35 @@ class TransformForest:
         return flat
 
     def to_gltf(self, mesh_index):
-        '''
-        Export a list of transforms as the 'nodes' section of a GLTF dict.
-
-        Will flatten tree.
+        """
+        Export a transforms as the 'nodes' section of a GLTF dict.
+        Flattens tree.
 
         Returns
         --------
         gltf: dict, with keys:
                   'nodes': list of dicts
-        '''
+        """
         gltf = collections.deque()
-
-        for node in self.nodes:
+        for node in self.nodes_geometry:
             if node == self.base_frame:
                 continue
             transform, geometry = self.get(frame_to=node,
                                            frame_from=self.base_frame)
             if geometry is None:
                 continue
-
             gltf.append({'matrix': transform.T.reshape(-1).tolist(),
                          'mesh': mesh_index[geometry],
                          'name': node})
-
         # we have flattened tree, so all nodes will be child of world
         gltf.appendleft({'name': self.base_frame,
                          'children': list(range(1, 1 + len(gltf)))})
         result = {'nodes': list(gltf)}
+
         return result
 
     def to_edgelist(self):
-        '''
+        """
         Export the current transforms as a list of edge tuples, with
         each tuple having the format:
         (node_a, node_b, {metadata})
@@ -129,7 +127,7 @@ class TransformForest:
         Returns
         -------
         edgelist: (n,) list of tuples
-        '''
+        """
         # wrapped in a list for nx 2.0
         export = list(nx.to_edgelist(self.transforms))
         for e in export:
@@ -142,17 +140,40 @@ class TransformForest:
 
     @util.cache_decorator
     def nodes(self):
-        return np.array(self.transforms.nodes())
+        """
+        A list of every node in the graph.
+
+        Returns
+        -------------
+        nodes: (n,) array, of node names
+        """
+        nodes = np.array(list(self.transforms.nodes()))
+        return nodes
 
     @util.cache_decorator
     def nodes_geometry(self):
-        nodes = [i for i in self.nodes if 'geometry' in self.transforms.node[i]]
-        return np.array(nodes)
+        """
+        The nodes in the scene graph with geometry attached.
+
+        Returns
+        ------------
+        nodes_geometry: (m,) array, of node names
+        """
+        nodes = self.transforms.nodes()
+        if len(nodes) == 0:
+            return np.array([])
+
+        nodes_geometry = []
+        for n in nodes:
+            if (n in self.transforms.node and
+                    'geometry' in self.transforms.node[n]):
+                nodes_geometry.append(n)
+        return np.array(nodes_geometry)
 
     def get(self,
             frame_to,
             frame_from=None):
-        '''
+        """
         Get the transform from one frame to another, assuming they are connected
         in the transform tree.
 
@@ -167,7 +188,7 @@ class TransformForest:
         Returns
         ---------
         transform:  (4,4) homogenous transformation matrix
-        '''
+        """
 
         if frame_from is None:
             frame_from = self.base_frame
@@ -197,16 +218,16 @@ class TransformForest:
         return transform, geometry
 
     def show(self):
-        '''
+        """
         Plot the graph layout of the scene.
-        '''
+        """
         import matplotlib.pyplot as plt
         nx.draw(self.transforms, with_labels=True)
         plt.show()
 
     def to_svg(self):
-        '''
-        '''
+        """
+        """
         from ..graph import graph_to_svg
         return graph_to_svg(self.transforms)
 
@@ -230,7 +251,7 @@ class TransformForest:
     def _get_path(self,
                   frame_from,
                   frame_to):
-        '''
+        """
         Find a path between two frames, either from cached paths or
         from the transform graph.
 
@@ -245,7 +266,7 @@ class TransformForest:
         ----------
         path: (n) list of frame keys
               eg, ['mesh_finger', 'mesh_hand', 'world']
-        '''
+        """
         key = (frame_from, frame_to)
         if not (key in self._paths):
             path = self.transforms.shortest_path_undirected(frame_from,
@@ -255,10 +276,10 @@ class TransformForest:
 
 
 class EnforcedForest(nx.DiGraph):
-    '''
+    """
     A subclass of networkx.DiGraph that will raise an error if an
     edge is added which would make the DiGraph not a forest or tree.
-    '''
+    """
 
     def __init__(self, *args, **kwargs):
         self.flags = {'strict': False,
@@ -338,16 +359,16 @@ class EnforcedForest(nx.DiGraph):
 
 
 def path_to_edges(path):
-    '''
+    """
     Turn an (n) path into a (2(n-1)) set of edges
-    '''
+    """
     return np.column_stack((path, path)).reshape(-1)[1:-1].reshape((-1, 2))
 
 
 def kwargs_to_matrix(**kwargs):
-    '''
+    """
     Turn a set of keyword arguments into a transformation matrix.
-    '''
+    """
     matrix = np.eye(4)
     if 'matrix' in kwargs:
         # a matrix takes precedence over other options
