@@ -40,7 +40,7 @@ class SceneViewer(pyglet.window.Window):
             # if you have a graphics card this will probably work
             conf = gl.Config(sample_buffers=1,
                              samples=4,
-                             depth_size=16,
+                             depth_size=24,
                              double_buffer=True)
             super(SceneViewer, self).__init__(config=conf,
                                               visible=visible,
@@ -99,8 +99,17 @@ class SceneViewer(pyglet.window.Window):
         self.vertex_list_mode[name] = gl.GL_LINES
 
     def _add_points(self, name, pointcloud):
+        """
+        Add a PointCloud object to the scene.
+
+        Parameters
+        ------------
+        name:       str, name of geometry
+        pointcloud: trimesh.points.PointCloud object
+        """
         self.vertex_list[name] = self.batch.add_indexed(
-            *points_to_vertex_list(pointcloud.vertices, pointcloud.vertices_color))
+            *points_to_vertex_list(pointcloud.vertices,
+                                   pointcloud.colors))
         self.vertex_list_md5[name] = geometry_md5(pointcloud)
         self.vertex_list_mode[name] = gl.GL_POINTS
 
@@ -146,18 +155,23 @@ class SceneViewer(pyglet.window.Window):
         max_depth = np.clip(max_depth, 500.00, np.inf)
         gl.glDepthRange(0.0, max_depth)
 
+        gl.glClearDepth(1.0)
+        gl.glEnable(gl.GL_DEPTH_TEST)
+        gl.glDepthFunc(gl.GL_LEQUAL)
+
         gl.glEnable(gl.GL_DEPTH_TEST)
         gl.glEnable(gl.GL_CULL_FACE)
         gl.glEnable(gl.GL_LIGHTING)
         gl.glEnable(gl.GL_LIGHT0)
         gl.glEnable(gl.GL_LIGHT1)
 
-        gl.glLightfv(gl.GL_LIGHT0, gl.GL_POSITION, _gl_vector(.5, .5, 1, 0))
+        # put the light at one corner of the scenes AABB
+        gl.glLightfv(gl.GL_LIGHT0,
+                     gl.GL_POSITION,
+                     _gl_vector(np.append(self.scene.bounds[1], 0)))
         gl.glLightfv(gl.GL_LIGHT0, gl.GL_SPECULAR, _gl_vector(.5, .5, 1, 1))
-        gl.glLightfv(gl.GL_LIGHT0, gl.GL_DIFFUSE, _gl_vector(1, 1, 1, 1))
-        gl.glLightfv(gl.GL_LIGHT1, gl.GL_POSITION, _gl_vector(1, 0, .5, 0))
-        gl.glLightfv(gl.GL_LIGHT1, gl.GL_DIFFUSE, _gl_vector(.5, .5, .5, 1))
-        gl.glLightfv(gl.GL_LIGHT1, gl.GL_SPECULAR, _gl_vector(1, 1, 1, 1))
+        gl.glLightfv(gl.GL_LIGHT0, gl.GL_DIFFUSE, _gl_vector(1, 1, 1, .75))
+        gl.glLightfv(gl.GL_LIGHT0, gl.GL_AMBIENT, _gl_vector(.1, .1, .1, .2))
 
         gl.glColorMaterial(gl.GL_FRONT_AND_BACK, gl.GL_AMBIENT_AND_DIFFUSE)
         gl.glEnable(gl.GL_COLOR_MATERIAL)
@@ -206,13 +220,24 @@ class SceneViewer(pyglet.window.Window):
             gl.glDisable(gl.GL_CULL_FACE)
 
     def on_resize(self, width, height):
+        try:
+            # for high DPI screens viewport size
+            # will be different then the passed size
+            width, height = self.get_viewport_size()
+        except BaseException:
+            # older versions of pyglet may not have this
+            pass
+        # set the new viewport size
         gl.glViewport(0, 0, width, height)
         gl.glMatrixMode(gl.GL_PROJECTION)
         gl.glLoadIdentity()
-        gl.gluPerspective(60., width / float(height), .01,
+        gl.gluPerspective(60.,
+                          width / float(height),
+                          .01,
                           self.scene.scale * 5.0)
         gl.glMatrixMode(gl.GL_MODELVIEW)
-        self.view['ball'].place([width / 2, height / 2], (width + height) / 2)
+        self.view['ball'].place([width / 2, height / 2],
+                                (width + height) / 2)
 
     def on_mouse_press(self, x, y, buttons, modifiers):
         self.view['ball'].down([x, -y])
@@ -346,7 +371,8 @@ def mesh_to_vertex_list(mesh, group=None):
     normals = mesh.vertex_normals.reshape(-1).tolist()
     faces = mesh.faces.reshape(-1).tolist()
     vertices = mesh.vertices.reshape(-1).tolist()
-    color_gl = _validate_colors(mesh.visual.vertex_colors, len(mesh.vertices))
+    color_gl = _validate_colors(mesh.visual.vertex_colors,
+                                len(mesh.vertices))
 
     args = (len(mesh.vertices),  # number of vertices
             gl.GL_TRIANGLES,    # mode
@@ -359,7 +385,9 @@ def mesh_to_vertex_list(mesh, group=None):
 
 
 def path_to_vertex_list(path, group=None):
-    vertices = path.vertices
+    if path.vertices.shape[1] == 2:
+        path = path.to_3D()
+
     lines = np.vstack([util.stack_lines(e.discrete(path.vertices))
                        for e in path.entities])
     index = np.arange(len(lines))
@@ -445,7 +473,7 @@ def _gl_vector(array, *args):
     return vector
 
 
-def render_scene(scene, resolution=(1080, 1080), visible=False, **kwargs):
+def render_scene(scene, resolution=(1080, 1080), visible=True, **kwargs):
     '''
     Render a preview of a scene to a PNG.
 
