@@ -1,5 +1,8 @@
-'''
-A data structure to hold visual information about meshes.
+"""
+visual.py
+-------------
+
+Hold and deal with visual information about meshes.
 
 There are lots of ways to encode visual information, and the goal of this
 architecture is to make it possible to define one, and then transparently
@@ -19,7 +22,7 @@ Rules
    a value should transparently change the mode. Color modes are:
      * vertex colors
      * face colors
-'''
+"""
 
 import numpy as np
 
@@ -27,20 +30,21 @@ import colorsys
 import collections
 
 from . import util
+from . import caching
 from . import grouping
 
 
 class ColorVisuals(object):
-    '''
+    """
     Store color information about a mesh.
-    '''
+    """
 
     def __init__(self,
                  mesh=None,
                  face_colors=None,
                  vertex_colors=None,
                  **kwargs):
-        '''
+        """
         Store color information about a mesh.
 
         Parameters
@@ -49,18 +53,19 @@ class ColorVisuals(object):
                        are associated with
         face_ colors:  (n,3|4) or (3,) or (4,) uint8, colors per-face
         vertex_colors: (n,3|4) or (3,) or (4,) uint8, colors per-vertex
-        '''
+        """
         self.mesh = mesh
-        self._data = util.DataStore()
-        self._cache = util.Cache(id_function=self.crc)
+        self._data = caching.DataStore()
+        self._cache = caching.Cache(id_function=self.crc)
 
-        self.defaults = {'material_diffuse': np.array([102, 102, 102, 255],
-                                                      dtype=np.uint8),
-                         'material_ambient': np.array([64, 64, 64, 255],
-                                                      dtype=np.uint8),
-                         'material_specular': np.array([197, 197, 197, 255],
-                                                       dtype=np.uint8),
-                         'material_shine': 77.0}
+        self.defaults = {
+            'material_diffuse': np.array([102, 102, 102, 255],
+                                         dtype=np.uint8),
+            'material_ambient': np.array([64, 64, 64, 255],
+                                         dtype=np.uint8),
+            'material_specular': np.array([197, 197, 197, 255],
+                                          dtype=np.uint8),
+            'material_shine': 77.0}
 
         if face_colors is not None:
             self.face_colors = face_colors
@@ -70,13 +75,13 @@ class ColorVisuals(object):
 
     @property
     def transparency(self):
-        '''
+        """
         Does the current object contain any transparency.
 
         Returns
         ----------
-        transparency: bool, does the current visual object contain transparency
-        '''
+        transparency: bool, does the current visual contain transparency
+        """
         if 'vertex_colors' in self._data:
             a_min = self._data['vertex_colors'][:, 3].min()
         elif 'face_colors' in self._data:
@@ -88,24 +93,26 @@ class ColorVisuals(object):
 
     @property
     def defined(self):
-        '''
+        """
         Are any colors defined for the current mesh.
 
         Returns
         ---------
         defined: bool, are colors defined or not.
-        '''
-        return self.kind is not None
+        """
+        defined = (not self._data.is_empty() and
+                   self.kind is not None)
+        return defined
 
     @property
     def kind(self):
-        '''
+        """
         What color mode has been set.
 
         Returns
         ----------
         mode: 'face', 'vertex', or None
-        '''
+        """
         self._verify_crc()
         if 'vertex_colors' in self._data:
             mode = 'vertex'
@@ -116,56 +123,26 @@ class ColorVisuals(object):
 
         return mode
 
-    def _verify_crc(self):
-        '''
-        Verify the checksums of cached face and vertex color, to verify
-        that a user hasn't altered them since they were generated from defaults.
-
-        If the colors have been altered since creation, move them into the DataStore
-        at self._data since the user action has made them user data.
-        '''
-        if len(self._cache) == 0:
-            return
-
-        for name in ['face', 'vertex']:
-            # the face or vertex colors
-            key_colors = str(name) + '_colors'
-            # the inital crc of the
-            key_crc = key_colors + '_crc'
-
-            if key_colors not in self._cache:
-                continue
-
-            colors = self._cache[key_colors]
-            # if the cached colors have been changed since creation we move
-            # them to data
-            if colors.crc() != self._cache[key_crc]:
-                # call the setter on the property using exec
-                # this avoids having to pass a setter to this function
-                if name == 'face':
-                    self.face_colors = colors
-                elif name == 'vertex':
-                    self.vertex_colors = colors
-                else:
-                    raise ValueError('unsupported name!!!')
-                self._cache.verify()
-
     def crc(self):
-        '''
+        """
         A checksum for the current visual object and its parent mesh.
 
         Returns
         ----------
         crc: int, checksum of data in visual object and its parent mesh
-        '''
+        """
+        # will make sure everything has been transferred
+        # to datastore that needs to be before returning crc
+
         result = self._data.crc()
         if hasattr(self.mesh, 'crc'):
-            result += self.mesh.crc()
+            # bitwise xor combines hashes better than a sum
+            result ^= self.mesh.crc()
         return result
 
     @property
     def face_colors(self):
-        '''
+        """
         Colors defined for each face of a mesh.
 
         If no colors are defined, defaults are returned.
@@ -173,12 +150,12 @@ class ColorVisuals(object):
         Returns
         ----------
         colors: (len(mesh.faces), 4) uint8, RGBA color for each face
-        '''
+        """
         return self._get_colors(name='face')
 
     @face_colors.setter
     def face_colors(self, values):
-        '''
+        """
         Set the colors for each face of a mesh.
 
         This will apply these colors and delete any previously specified
@@ -190,7 +167,7 @@ class ColorVisuals(object):
                 (len(mesh.faces), 4), set each face to the specified color
                 (3,) int, set the whole mesh this color
                 (4,) int, set the whole mesh this color
-        '''
+        """
         colors = to_rgba(values)
 
         if (self.mesh is not None and
@@ -205,18 +182,18 @@ class ColorVisuals(object):
 
     @property
     def vertex_colors(self):
-        '''
+        """
         Return the colors for each vertex of a mesh
 
         Returns
         ------------
-        colors: (len(mesh.vertices), 4) uint8, color for each vertex of mesh.
-        '''
+        colors: (len(mesh.vertices), 4) uint8, color for each vertex
+        """
         return self._get_colors(name='vertex')
 
     @vertex_colors.setter
     def vertex_colors(self, values):
-        '''
+        """
         Set the colors for each vertex of a mesh
 
         This will apply these colors and delete any previously specified
@@ -224,11 +201,11 @@ class ColorVisuals(object):
 
         Parameters
         ------------
-        colors: (len(mesh.vertices), 3), set each face to the specified color
-                (len(mesh.vertices), 4), set each face to the specified color
+        colors: (len(mesh.vertices), 3), set each face to the color
+                (len(mesh.vertices), 4), set each face to the color
                 (3,) int, set the whole mesh this color
                 (4,) int, set the whole mesh this color
-        '''
+        """
         # make sure passed values are numpy array
         values = np.asanyarray(values)
         # Ensure the color shape is sane
@@ -252,7 +229,7 @@ class ColorVisuals(object):
 
     def _get_colors(self,
                     name):
-        '''
+        """
         A magical function which maintains the sanity of vertex and face colors.
 
         * If colors have been explicitly stored or changed, they are considered
@@ -272,7 +249,7 @@ class ColorVisuals(object):
         Returns
         -----------
         colors: (count, 4) uint8, RGBA colors
-        '''
+        """
 
         try:
             counts = {'face': len(self.mesh.faces),
@@ -287,13 +264,13 @@ class ColorVisuals(object):
         key_crc = key_colors + '_crc'
 
         if key_colors in self._data:
-            # if a user has explicitly stored or changed the color it will be
-            # in data
+            # if a user has explicitly stored or changed the color it
+            # will be in data
             return self._data[key_colors]
 
         elif key_colors in self._cache:
-            # if the colors have been autogenerated already they will be in the
-            # cache
+            # if the colors have been autogenerated already they
+            # will be in the cache
             colors = self._cache[key_colors]
             # if the cached colors have been changed since creation we move
             # them to data
@@ -307,22 +284,23 @@ class ColorVisuals(object):
                 else:
                     raise ValueError('unsupported name!!!')
                 self._cache.verify()
-
         else:
             # colors have never been accessed
             if self.kind is None:
-                # no colors are defined, so create a (count, 4) tiled copy of
-                # the default color
+                # no colors are defined, so create a (count, 4) tiled
+                # copy of the default color
                 colors = np.tile(self.defaults['material_diffuse'],
                                  (count, 1))
             elif (self.kind == 'vertex' and
                   name == 'face'):
-                colors = vertex_to_face_color(vertex_colors=self.vertex_colors,
-                                              faces=self.mesh.faces)
+                colors = vertex_to_face_color(
+                    vertex_colors=self.vertex_colors,
+                    faces=self.mesh.faces)
             elif (self.kind == 'face' and
                   name == 'vertex'):
-                colors = face_to_vertex_color(mesh=self.mesh,
-                                              face_colors=self.face_colors)
+                colors = face_to_vertex_color(
+                    mesh=self.mesh,
+                    face_colors=self.face_colors)
             else:
                 raise ValueError('self.kind not accepted values!!')
 
@@ -330,28 +308,62 @@ class ColorVisuals(object):
                 colors.shape != (count, 4)):
             raise ValueError('face colors incorrect shape!')
 
-        # subclass the array to track for changes using a zlib.adler32 CRC
-        colors = util.tracked_array(colors)
-        # put the generated colors and their initial checksum into the cache
+        # subclass the array to track for changes using a CRC
+        colors = caching.tracked_array(colors)
+        # put the generated colors and their initial checksum into cache
         self._cache[key_colors] = colors
         self._cache[key_crc] = colors.crc()
 
         return colors
 
+    def _verify_crc(self):
+        """
+        Verify the checksums of cached face and vertex color, to verify
+        that a user hasn't altered them since they were generated from
+        defaults.
+
+        If the colors have been altered since creation, move them into
+        the DataStore at self._data since the user action has made them
+        user data.
+        """
+        if not hasattr(self, '_cache') or len(self._cache) == 0:
+            return
+
+        for name in ['face', 'vertex']:
+            # the face or vertex colors
+            key_colors = str(name) + '_colors'
+            # the inital crc of the
+            key_crc = key_colors + '_crc'
+
+            if key_colors not in self._cache:
+                continue
+
+            colors = self._cache[key_colors]
+            # if the cached colors have been changed since creation
+            # move them to data
+            if colors.crc() != self._cache[key_crc]:
+                if name == 'face':
+                    self.face_colors = colors
+                elif name == 'vertex':
+                    self.vertex_colors = colors
+                else:
+                    raise ValueError('unsupported name!!!')
+                self._cache.verify()
+
     def update_vertices(self, mask):
-        '''
+        """
         Apply a mask to remove or duplicate vertex properties.
-        '''
+        """
         self._update_key(mask, 'vertex_colors')
 
     def update_faces(self, mask):
-        '''
+        """
         Apply a mask to remove or duplicate face properties
-        '''
+        """
         self._update_key(mask, 'face_colors')
 
     def face_subset(self, face_index):
-        '''
+        """
         Given a mask of face indices, return a sliced version.
 
         Parameters
@@ -361,13 +373,45 @@ class ColorVisuals(object):
 
         Returns
         ----------
-        visual: ColorVisuals object containing information about a subset of faces.
-        '''
-        result = ColorVisuals(face_colors=self.face_colors[face_index])
+        visual: ColorVisuals object containing a subset of faces.
+        """
+        if self.defined:
+            result = ColorVisuals(
+                face_colors=self.face_colors[face_index])
+        else:
+            result = ColorVisuals()
+
         return result
 
+    @property
+    def main_color(self):
+        """
+        What is the most commonly occuring color.
+
+        Returns
+        ------------
+        color: (4,) uint8, most common color
+        """
+        if self.kind is None:
+            return DEFAULT_COLOR
+        elif self.kind == 'face':
+            colors = self.face_colors
+        elif self.kind == 'vertex':
+            colors = self.vertex_colors
+        else:
+            raise ValueError('color kind incorrect!')
+
+        # find the unique colors
+        unique, inverse = grouping.unique_rows(colors)
+        # the most commonly occuring color, or mode
+        # this will be an index of inverse, not colors
+        mode_index = np.bincount(inverse).argmax()
+        color = colors[unique[mode_index]]
+
+        return color
+
     def concatenate(self, other, *args):
-        '''
+        """
         Concatenate two or more ColorVisuals objects into a single object.
 
         Parameters
@@ -379,12 +423,12 @@ class ColorVisuals(object):
         -----------
         result: ColorVisuals object containing information from current
                 object and others in the order it was passed.
-        '''
+        """
         result = concatenate_visuals(self, other, *args)
         return result
 
     def __add__(self, other):
-        '''
+        """
         Concatenate two ColorVisuals objects into a single object.
 
         Parameters
@@ -395,11 +439,11 @@ class ColorVisuals(object):
         -----------
         result: ColorVisuals object containing information from current
                 object and other in the order (self, other)
-        '''
+        """
         return self.concatenate(other)
 
     def _update_key(self, mask, key):
-        '''
+        """
         Mask the value contained in the DataStore at a specified key.
 
         Parameters
@@ -407,14 +451,14 @@ class ColorVisuals(object):
         mask: (n,) int
               (n,) bool
         key: hashable object, in self._data
-        '''
+        """
         mask = np.asanyarray(mask)
         if key in self._data:
             self._data[key] = self._data[key][mask]
 
 
 def create_visual(**kwargs):
-    '''
+    """
     Create Visuals object from keyword arguments.
 
     Parameters
@@ -426,12 +470,12 @@ def create_visual(**kwargs):
     Returns
     ----------
     visuals: ColorVisuals object.
-    '''
+    """
     return ColorVisuals(**kwargs)
 
 
 def to_rgba(colors, dtype=np.uint8):
-    '''
+    """
     Convert a single or multiple RGB colors to RGBA colors.
 
     Parameters
@@ -442,7 +486,7 @@ def to_rgba(colors, dtype=np.uint8):
     ----------
     colors: (n,4) list of RGBA colors
             (4,)  single RGBA color
-    '''
+    """
     if not util.is_sequence(colors):
         return
     # colors as numpy array
@@ -473,8 +517,30 @@ def to_rgba(colors, dtype=np.uint8):
     return colors
 
 
+def hex_to_rgba(color):
+    """
+    Turn a string hex color to a (4,) RGBA color.
+
+    Parameters
+    -----------
+    color: str, hex color
+
+    Returns
+    -----------
+    rgba: (4,) np.uint8, RGBA color
+    """
+    value = str(color).lstrip('#').strip()
+    if len(value) == 6:
+        rgb = [int(value[i:i + 2], 16) for i in (0, 2, 4)]
+        rgba = np.append(rgb, 255).astype(np.uint8)
+    else:
+        raise ValueError('Only RGB supported')
+
+    return rgba
+
+
 def concatenate_visuals(visuals, *args):
-    '''
+    """
     Concatenate multiple visual objects.
 
     Parameters
@@ -485,7 +551,7 @@ def concatenate_visuals(visuals, *args):
     Returns
     ----------
     concat: ColorVisuals object
-    '''
+    """
     # get a flat list of ColorVisuals objects
     visuals = np.append(visuals, args)
 
@@ -512,7 +578,7 @@ def concatenate_visuals(visuals, *args):
 
 
 def random_color(dtype=np.uint8):
-    '''
+    """
     Return a random RGB color using datatype specified.
 
     Parameters
@@ -522,7 +588,7 @@ def random_color(dtype=np.uint8):
     Returns
     ----------
     color: (4,) dtype, random color that looks OK
-    '''
+    """
     hue = np.random.random() + .61803
     hue %= 1.0
     color = np.array(colorsys.hsv_to_rgb(hue, .99, .99))
@@ -534,7 +600,7 @@ def random_color(dtype=np.uint8):
 
 
 def vertex_to_face_color(vertex_colors, faces):
-    '''
+    """
     Convert a list of vertex colors to face colors.
 
     Parameters
@@ -545,14 +611,14 @@ def vertex_to_face_color(vertex_colors, faces):
     Returns
     -----------
     face_colors: (m,4) colors
-    '''
+    """
     vertex_colors = to_rgba(vertex_colors)
     face_colors = vertex_colors[faces].mean(axis=1)
     return face_colors.astype(np.uint8)
 
 
 def face_to_vertex_color(mesh, face_colors, dtype=np.uint8):
-    '''
+    """
     Convert a list of face colors into a list of vertex colors.
 
     Parameters
@@ -564,11 +630,10 @@ def face_to_vertex_color(mesh, face_colors, dtype=np.uint8):
     Returns
     -----------
     vertex_colors: (m,4) dtype, colors for each vertex
-    '''
-
+    """
     rgba = to_rgba(face_colors)
-
-    vertex_colors = mesh.faces_sparse.dot(face_colors.astype(np.float64))
+    vertex_colors = mesh.faces_sparse.dot(
+        rgba.astype(np.float64))
     vertex_colors /= mesh.faces_sparse.sum(axis=1)
     vertex_colors = vertex_colors.astype(dtype)
 
@@ -576,7 +641,7 @@ def face_to_vertex_color(mesh, face_colors, dtype=np.uint8):
 
 
 def colors_to_materials(colors, count=None):
-    '''
+    """
     Convert a list of colors into a list of unique materials and material
     indexes.
 
@@ -589,7 +654,7 @@ def colors_to_materials(colors, count=None):
     -----------
     diffuse: (m,4) int, colors
     index:   (count,) int, index of each color
-    '''
+    """
 
     # convert RGB to RGBA
     rgba = to_rgba(colors)
